@@ -1,18 +1,17 @@
 //! The client-side proxy API.
 
 use enumflags2::BitFlags;
-use futures_util::StreamExt;
-use static_assertions::assert_impl_all;
+use futures_lite::StreamExt;
 use std::{fmt, ops::Deref};
 use zbus_names::{BusName, InterfaceName, MemberName, UniqueName};
 use zvariant::{ObjectPath, OwnedValue, Value};
 
 use crate::{
+    Error, Result,
     blocking::Connection,
     message::Message,
     proxy::{Defaults, MethodFlags},
     utils::block_on,
-    Error, Result,
 };
 
 use crate::fdo;
@@ -51,15 +50,14 @@ pub use builder::Builder;
 ///
 /// # Note
 ///
-/// It is recommended to use the [`proxy`] macro, which provides a more convenient and
-/// type-safe *façade* `Proxy` derived from a Rust trait.
+/// It is recommended to use the [`macro@crate::proxy`] macro, which provides a more
+/// convenient and type-safe *façade* `Proxy` derived from a Rust trait.
 ///
 /// ## Current limitations:
 ///
 /// At the moment, `Proxy` doesn't prevent [auto-launching][al].
 ///
-/// [`proxy`]: attr.proxy.html
-/// [al]: https://github.com/dbus2/zbus/issues/54
+/// [al]: https://github.com/z-galaxy/zbus/issues/54
 #[derive(Clone)]
 pub struct Proxy<'a> {
     conn: Connection,
@@ -77,8 +75,6 @@ impl fmt::Debug for Proxy<'_> {
     }
 }
 
-assert_impl_all!(Proxy<'_>: Send, Sync, Unpin);
-
 impl<'a> Proxy<'a> {
     /// Create a new `Proxy` for the given destination/path/interface.
     pub fn new<D, P, I>(
@@ -328,14 +324,7 @@ impl<'a> Proxy<'a> {
             .map(SignalIterator)
     }
 
-    /// Get an iterator to receive owner changed events.
-    ///
-    /// If the proxy destination is a unique name, the stream will be notified of the peer
-    /// disconnection from the bus (with a `None` value).
-    ///
-    /// If the proxy destination is a well-known name, the stream will be notified whenever the name
-    /// owner is changed, either by a new peer being granted ownership (`Some` value) or when the
-    /// name is released (with a `None` value).
+    /// Get an iterator to receive property changed events.
     ///
     /// Note that zbus doesn't queue the updates. If the listener is slower than the receiver, it
     /// will only receive the last update.
@@ -346,7 +335,14 @@ impl<'a> Proxy<'a> {
         PropertyIterator(block_on(self.inner().receive_property_changed(name)))
     }
 
-    /// Get an iterator to receive property changed events.
+    /// Get an iterator to receive owner changed events.
+    ///
+    /// If the proxy destination is a unique name, the stream will be notified of the peer
+    /// disconnection from the bus (with a `None` value).
+    ///
+    /// If the proxy destination is a well-known name, the stream will be notified whenever the name
+    /// owner is changed, either by a new peer being granted ownership (`Some` value) or when the
+    /// name is released (with a `None` value).
     ///
     /// Note that zbus doesn't queue the updates. If the listener is slower than the receiver, it
     /// will only receive the last update.
@@ -407,8 +403,6 @@ impl<'a> SignalIterator<'a> {
     }
 }
 
-assert_impl_all!(SignalIterator<'_>: Send, Sync, Unpin);
-
 impl std::iter::Iterator for SignalIterator<'_> {
     type Item = Message;
 
