@@ -3,7 +3,7 @@
 //! The D-Bus specification defines the message bus messages and some standard interfaces that may
 //! be useful across various D-Bus applications. This module provides their proxy.
 
-use enumflags2::{bitflags, BitFlags};
+use enumflags2::{BitFlags, bitflags};
 use serde::{Deserialize, Serialize};
 use serde_repr::{Deserialize_repr, Serialize_repr};
 use std::collections::HashMap;
@@ -12,35 +12,30 @@ use zbus_names::{
 };
 #[cfg(unix)]
 use zvariant::OwnedFd;
-use zvariant::{as_value::optional, Optional, Type};
+use zvariant::{DeserializeDict, Optional, SerializeDict, Type};
 
 use super::Result;
-use crate::{proxy, OwnedGuid};
+use crate::{OwnedGuid, proxy};
 
-/// The flags used by the bus [`request_name`] method.
+/// The flags used by the [`DBusProxy::request_name`] method.
 ///
-/// [`request_name`]: struct.DBusProxy.html#method.request_name
-#[bitflags]
+/// The default flags (returned by [`BitFlags::default`]) are `AllowReplacement`, `ReplaceExisting`,
+/// and `DoNotQueue`.
+#[bitflags(default = AllowReplacement | ReplaceExisting | DoNotQueue)]
 #[repr(u32)]
 #[derive(Type, Debug, PartialEq, Eq, Copy, Clone, Serialize, Deserialize)]
 pub enum RequestNameFlags {
     /// If an application A specifies this flag and succeeds in becoming the owner of the name, and
-    /// another application B later calls [`request_name`] with the [`ReplaceExisting`] flag, then
-    /// application A will lose ownership and receive a `org.freedesktop.DBus.NameLost` signal, and
-    /// application B will become the new owner. If [`AllowReplacement`] is not specified by
-    /// application A, or [`ReplaceExisting`] is not specified by application B, then application B
-    /// will not replace application A as the owner.
-    ///
-    /// [`ReplaceExisting`]: enum.RequestNameFlags.html#variant.ReplaceExisting
-    /// [`AllowReplacement`]: enum.RequestNameFlags.html#variant.AllowReplacement
-    /// [`request_name`]: struct.DBusProxy.html#method.request_name
+    /// another application B later calls [`DBusProxy::request_name`] with the `ReplaceExisting`
+    /// flag, then application A will lose ownership and receive a `org.freedesktop.DBus.NameLost`
+    /// signal, and application B will become the new owner. If `AllowReplacement` is not specified
+    /// by application A, or `ReplaceExisting` is not specified by application B, then application
+    /// B will not replace application A as the owner.
     AllowReplacement = 0x01,
     /// Try to replace the current owner if there is one. If this flag is not set the application
     /// will only become the owner of the name if there is no current owner. If this flag is set,
     /// the application will replace the current owner if the current owner specified
-    /// [`AllowReplacement`].
-    ///
-    /// [`AllowReplacement`]: enum.RequestNameFlags.html#variant.AllowReplacement
+    /// `AllowReplacement`.
     ReplaceExisting = 0x02,
     /// Without this flag, if an application requests a name that is already owned, the
     /// application will be placed in a queue to own the name when the current owner gives it
@@ -52,42 +47,27 @@ pub enum RequestNameFlags {
     DoNotQueue = 0x04,
 }
 
-/// The return code of the [`request_name`] method.
-///
-/// [`request_name`]: struct.DBusProxy.html#method.request_name
+/// The return code of the [`DBusProxy::request_name`] method.
 #[repr(u32)]
 #[derive(Deserialize_repr, Serialize_repr, Type, Debug, PartialEq, Eq)]
 pub enum RequestNameReply {
     /// The caller is now the primary owner of the name, replacing any previous owner. Either the
-    /// name had no owner before, or the caller specified [`ReplaceExisting`] and the current owner
-    /// specified [`AllowReplacement`].
-    ///
-    /// [`ReplaceExisting`]: enum.RequestNameFlags.html#variant.ReplaceExisting
-    /// [`AllowReplacement`]: enum.RequestNameFlags.html#variant.AllowReplacement
+    /// name had no owner before, or the caller specified [`RequestNameFlags::ReplaceExisting`] and
+    /// the current owner specified [`RequestNameFlags::AllowReplacement`].
     PrimaryOwner = 0x01,
-    /// The name already had an owner, [`DoNotQueue`] was not specified, and either the current
-    /// owner did not specify [`AllowReplacement`] or the requesting application did not specify
-    /// [`ReplaceExisting`].
-    ///
-    /// [`DoNotQueue`]: enum.RequestNameFlags.html#variant.DoNotQueue
-    /// [`ReplaceExisting`]: enum.RequestNameFlags.html#variant.ReplaceExisting
-    /// [`AllowReplacement`]: enum.RequestNameFlags.html#variant.AllowReplacement
+    /// The name already had an owner, [`RequestNameFlags::DoNotQueue`] was not specified, and
+    /// either the current owner did not specify [`RequestNameFlags::AllowReplacement`] or the
+    /// requesting application did not specify [`RequestNameFlags::ReplaceExisting`].
     InQueue = 0x02,
-    /// The name already had an owner, [`DoNotQueue`] was specified, and either
-    /// [`AllowReplacement`] was not specified by the current owner, or [`ReplaceExisting`] was
-    /// not specified by the requesting application.
-    ///
-    /// [`DoNotQueue`]: enum.RequestNameFlags.html#variant.DoNotQueue
-    /// [`ReplaceExisting`]: enum.RequestNameFlags.html#variant.ReplaceExisting
-    /// [`AllowReplacement`]: enum.RequestNameFlags.html#variant.AllowReplacement
+    /// The name already had an owner, [`RequestNameFlags::DoNotQueue`] was specified, and either
+    /// [`RequestNameFlags::AllowReplacement`] was not specified by the current owner, or
+    /// [`RequestNameFlags::ReplaceExisting`] was not specified by the requesting application.
     Exists = 0x03,
     /// The application trying to request ownership of a name is already the owner of it.
     AlreadyOwner = 0x04,
 }
 
-/// The return code of the [`release_name`] method.
-///
-/// [`release_name`]: struct.DBusProxy.html#method.release_name
+/// The return code of the [`DBusProxy::release_name`] method.
 #[repr(u32)]
 #[derive(Deserialize_repr, Serialize_repr, Type, Debug, PartialEq, Eq)]
 pub enum ReleaseNameReply {
@@ -103,6 +83,33 @@ pub enum ReleaseNameReply {
     NotOwner = 0x03,
 }
 
+/// The return code of the [`DBusProxy::start_service_by_name`] method.
+///
+/// In zbus 6.0, this will become the return type of `start_service_by_name`.
+/// For now, it's provided separately with a `TryFrom<u32>` implementation
+/// to avoid breaking changes in the API.
+#[repr(u32)]
+#[derive(Deserialize_repr, Serialize_repr, Type, Debug, PartialEq, Eq)]
+pub enum StartServiceReply {
+    /// The service was successfully started.
+    Success = 0x01,
+    /// The service was already running.
+    AlreadyRunning = 0x02,
+}
+
+// FIXME: When releasing 6.0, use StartServiceReply directly in start_service_by_name instead
+impl TryFrom<u32> for StartServiceReply {
+    type Error = super::Error;
+
+    fn try_from(value: u32) -> Result<Self> {
+        match value {
+            0x01 => Ok(StartServiceReply::Success),
+            0x02 => Ok(StartServiceReply::AlreadyRunning),
+            _ => Err(super::Error::ZBus(crate::Error::InvalidReply)),
+        }
+    }
+}
+
 /// Credentials of a process connected to a bus server.
 ///
 /// If unable to determine certain credentials (for instance, because the process is not on the same
@@ -112,51 +119,26 @@ pub enum ReleaseNameReply {
 ///
 /// **Note**: unknown keys, in particular those with "." that are not from the specification, will
 /// be ignored. Use your own implementation or contribute your keys here, or in the specification.
-#[derive(Debug, Default, Deserialize, PartialEq, Eq, Serialize, Type)]
+#[derive(Debug, Default, DeserializeDict, PartialEq, Eq, SerializeDict, Type)]
 #[zvariant(signature = "a{sv}")]
-#[serde(default)]
 pub struct ConnectionCredentials {
-    #[serde(
-        rename = "UnixUserID",
-        with = "optional",
-        skip_serializing_if = "Option::is_none"
-    )]
+    #[zvariant(rename = "UnixUserID")]
     pub(crate) unix_user_id: Option<u32>,
 
-    #[serde(
-        rename = "UnixGroupIDs",
-        with = "optional",
-        skip_serializing_if = "Option::is_none"
-    )]
+    #[zvariant(rename = "UnixGroupIDs")]
     pub(crate) unix_group_ids: Option<Vec<u32>>,
 
     #[cfg(unix)]
-    #[serde(
-        rename = "ProcessFD",
-        with = "optional",
-        skip_serializing_if = "Option::is_none"
-    )]
+    #[zvariant(rename = "ProcessFD")]
     pub(crate) process_fd: Option<OwnedFd>,
 
-    #[serde(
-        rename = "ProcessID",
-        with = "optional",
-        skip_serializing_if = "Option::is_none"
-    )]
+    #[zvariant(rename = "ProcessID")]
     pub(crate) process_id: Option<u32>,
 
-    #[serde(
-        rename = "WindowsSID",
-        with = "optional",
-        skip_serializing_if = "Option::is_none"
-    )]
+    #[zvariant(rename = "WindowsSID")]
     pub(crate) windows_sid: Option<String>,
 
-    #[serde(
-        rename = "LinuxSecurityLabel",
-        with = "optional",
-        skip_serializing_if = "Option::is_none"
-    )]
+    #[zvariant(rename = "LinuxSecurityLabel")]
     pub(crate) linux_security_label: Option<Vec<u8>>,
 }
 
@@ -402,3 +384,16 @@ pub trait DBus {
     #[zbus(property)]
     fn interfaces(&self) -> Result<Vec<OwnedInterfaceName>>;
 }
+
+#[cfg(test)]
+mod test {
+    use super::*;
+
+    #[test]
+    fn request_name_flags_default() {
+        let flags = BitFlags::<RequestNameFlags>::default();
+        assert!(flags.contains(RequestNameFlags::AllowReplacement));
+        assert!(flags.contains(RequestNameFlags::ReplaceExisting));
+        assert!(flags.contains(RequestNameFlags::DoNotQueue));
+    }
+}
