@@ -5,17 +5,16 @@ use std::{
     str::FromStr,
 };
 
-use serde::{de, Deserialize, Serialize};
+use serde::{Deserialize, Serialize, de};
 use zvariant::{Str, Type};
 
 /// A D-Bus server GUID.
 ///
 /// See the D-Bus specification [UUIDs chapter] for details.
 ///
-/// You can create a `Guid` from an existing string with [`Guid::try_from::<&str>`][TryFrom].
+/// You can create a `Guid` from an existing string with [`Guid::try_from::<&str>`].
 ///
 /// [UUIDs chapter]: https://dbus.freedesktop.org/doc/dbus-specification.html#uuids
-/// [TryFrom]: #impl-TryFrom%3C%26%27_%20str%3E
 #[derive(Clone, Debug, PartialEq, Eq, Hash, Type, Serialize)]
 pub struct Guid<'g>(Str<'g>);
 
@@ -206,7 +205,7 @@ impl From<Guid<'_>> for OwnedGuid {
 
 impl From<OwnedGuid> for Str<'_> {
     fn from(value: OwnedGuid) -> Self {
-        value.0 .0
+        value.0.0
     }
 }
 
